@@ -42,8 +42,7 @@ from .helpers import (
     temp_to_hex,
 )
 from .opentherm import parity
-
-# from .ramses import RAMSES_CODES
+from .ramses import RAMSES_CODES
 
 COMMAND_FORMAT = "{:<2} {} {} {} {} {} {:03d} {}"
 
@@ -106,15 +105,46 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
+def _pkt_header_idx(pkt: str, rx_header=None) -> Optional[str]:  # TODO:
+    """Return the index/ordinal of a packet."""
+
+    verb = pkt[4:6]
+    code = pkt[41:45]
+    payload = pkt[50:]
+
+    # if is_array:
+    #     return ""
+
+    if code in ("0005", "000C"):  # zone_idx, device_class
+        return payload[:4]
+
+    if code == "0404":  # zone_schedule: zone_idx, frag_idx
+        return payload[:2] + payload[10:12]
+
+    if code == "0418":  # fault_log: log_idx
+        return None if payload == CODE_SCHEMA["0418"]["null_rp"] else payload[4:6]
+
+    if code in ("3150",):  # 1FC9 can too
+        return payload[:2]
+
+    if code == "3220":  #
+        return payload[4:6]  # ot_msg_id (is ot_msg_type needed to?)
+
+    try:
+        if RAMSES_CODES[code][verb][:11] == r"^0[0-9A-F]":
+            return None if code in ("0016", "xx") else payload[:2]
+    except KeyError:
+        return
+
+
 def _pkt_header(pkt: str, rx_header=None) -> Optional[str]:
     """Return the QoS header of a packet."""
 
     verb = pkt[4:6]
     src, dst, _ = extract_addrs(pkt[11:40])
     code = pkt[41:45]
-    payload = pkt[50:]
 
-    if code == "1FC9":
+    if code == "1FC9":  # TODO: will need to do something similar for 3220?
         if src == dst:
             return "|".join((W_, dst.id, code) if rx_header else (I_, src.id, code))
         if verb == W_:
@@ -127,27 +157,17 @@ def _pkt_header(pkt: str, rx_header=None) -> Optional[str]:
             return
         verb = RP if verb == RQ else I_  # RQ/RP, or W/I
 
+    if code in ("0001", "7FFF") and rx_header:  # code has no RQ, no W
+        return
+
     addr = dst if src.type == "18" else src
     header = "|".join((verb, addr.id, code))
 
-    if code in ("0001", "7FFF") and rx_header:  # code has no no RQ, no W
-        return
+    header_idx = _pkt_header_idx(pkt)
+    if header_idx:
+        header += f"|{header_idx}"
 
-    if code in ("0005", "000C"):  # zone_idx, device_class
-        return "|".join((header, payload[:4]))
-
-    if code == "0404":  # zone_schedule: zone_idx, frag_idx
-        return "|".join((header, payload[:2] + payload[10:12]))
-
-    if code == "0418":  # fault_log: log_idx
-        if payload == CODE_SCHEMA["0418"]["null_rp"]:
-            return header
-        return "|".join((header, payload[4:6]))
-
-    if code in ("1F09", "1FC9", "2E04"):  # have no domain_id (1FC9 can)
-        return header
-
-    return "|".join((header, payload[:2]))  # assume has a domain_id
+    return header
 
 
 def validate_system_args(fcn):
@@ -313,7 +333,7 @@ class Command:
         """Constructor to get the mode of the DHW (c.f. parser_1f41)."""
         return cls(RQ, "1F41", "00", ctl_id, **kwargs)
 
-    @classmethod  # constructor for RQ/1F41
+    @classmethod  # constructor for W_/1F41
     @validate_system_args
     def set_dhw_mode(
         cls, ctl_id: str, mode=None, active: bool = None, until=None, **kwargs
@@ -344,15 +364,26 @@ class Command:
 
         assert mode in ZONE_MODE_LOOKUP, mode
 
-        payload = "00"
-        payload += "01" if bool(active) else "00"
-        payload += ZONE_MODE_LOOKUP[mode] + "FFFFFF"
-        payload += "" if until is None else dtm_to_hex(until)
+        payload = "".join(
+            (
+                "00",
+                "01" if bool(active) else "00",
+                ZONE_MODE_LOOKUP[mode],
+                "FFFFFF",  # duration?
+                "" if until is None else dtm_to_hex(until),
+            )
+        )
 
         return cls(W_, "1F41", payload, ctl_id, **kwargs)
 
     @classmethod  # constructor for RQ/10A0
     @validate_system_args
+    def get_dhw_params(cls, ctl_id: str, **kwargs):
+        """Constructor to get the params of the DHW (c.f. parser_10a0)."""
+        return cls(RQ, "10A0", "00", ctl_id, **kwargs)
+
+    @classmethod  # constructor for W_/10A0
+    @validate_system_args
     def set_dhw_params(
         cls,
         ctl_id: str,
@@ -388,6 +419,12 @@ class Command:
         payload = f"0023000800{frag_idx + 1:02X}{frag_cnt:02X}"
         return cls(RQ, "0404", payload, ctl_id, **kwargs)
 
+    @classmethod  # constructor for RQ/1260
+    @validate_system_args
+    def get_dhw_temp(cls, ctl_id: str, **kwargs):
+        """Constructor to get the temperature of the DHW sensor (c.f. parser_10a0)."""
+        return cls(RQ, "1260", "00", ctl_id, **kwargs)
+
     @classmethod  # constructor for RQ/3220
     def get_opentherm_data(cls, dev_id: str, msg_id: int, **kwargs):
         """Constructor to get (Read-Data) opentherm msg value (c.f. parser_3220)."""
@@ -476,8 +513,6 @@ class Command:
     ):
         """Constructor to set the TPI params of a system (c.f. parser_1100)."""
 
-        payload = f"{domain_id:02X}" if isinstance(domain_id, int) else domain_id
-
         assert cycle_rate is None or cycle_rate in (3, 6, 9, 12), cycle_rate
         assert min_on_time is None or 1 <= min_on_time <= 5, min_on_time
         assert min_off_time is None or 1 <= min_off_time <= 5, min_off_time
@@ -485,10 +520,15 @@ class Command:
             proportional_band_width is None or 1.5 <= proportional_band_width <= 3.0
         ), proportional_band_width
 
-        payload += f"{cycle_rate * 4:02X}"
-        payload += f"{int(min_on_time * 4):02X}"
-        payload += f"{int(min_off_time * 4):02X}FF"
-        payload += f"{temp_to_hex(proportional_band_width)}01"
+        payload = "".join(
+            (
+                f"{domain_id:02X}" if isinstance(domain_id, int) else domain_id,
+                f"{cycle_rate * 4:02X}",
+                f"{int(min_on_time * 4):02X}",
+                f"{int(min_off_time * 4):02X}FF",
+                f"{temp_to_hex(proportional_band_width)}01",
+            )
+        )
 
         return cls(W_, "1100", payload, ctl_id, **kwargs)
 
@@ -523,10 +563,14 @@ class Command:
         bitmap |= 0 if openwindow_function else 2
         bitmap |= 0 if multiroom_mode else 16
 
-        payload = f"{zone_idx:02X}"
-        payload += f"{bitmap:02X}"
-        payload += temp_to_hex(min_temp)
-        payload += temp_to_hex(max_temp)
+        payload = "".join(
+            (
+                f"{zone_idx:02X}",
+                f"{bitmap:02X}",
+                temp_to_hex(min_temp),
+                temp_to_hex(max_temp),
+            )
+        )
 
         return cls(W_, "000A", payload, ctl_id, **kwargs)
 
@@ -555,12 +599,16 @@ class Command:
         assert 0 <= valve_run_time <= 240, valve_run_time
         assert 0 <= pump_run_time <= 99, pump_run_time
 
-        payload = f"{zone_idx:02X}"
-        payload += f"C801{max_flow_setpoint:02X}"
-        payload += f"C901{min_flow_setpoint:02X}"
-        payload += f"CA01{valve_run_time:02X}"
-        payload += f"CB01{pump_run_time:02X}"
-        payload += f"CC01{1:02X}"
+        payload = "".join(
+            (
+                f"{zone_idx:02X}",
+                f"C801{max_flow_setpoint:02X}",
+                f"C901{min_flow_setpoint:02X}",
+                f"CA01{valve_run_time:02X}",
+                f"CB01{pump_run_time:02X}",
+                f"CC01{1:02X}",
+            )
+        )
 
         return cls(W_, "1030", payload, ctl_id, **kwargs)
 
@@ -629,11 +677,15 @@ class Command:
 
         assert mode in ZONE_MODE_LOOKUP, mode
 
-        payload = f"{zone_idx:02X}"
-        payload += temp_to_hex(setpoint)  # None means max, if a temp is required
-        payload += ZONE_MODE_LOOKUP[mode]
-        payload += "FFFFFF" if duration is None else f"{duration:06X}"
-        payload += "" if until is None else dtm_to_hex(until)
+        payload = "".join(
+            (
+                f"{zone_idx:02X}",
+                temp_to_hex(setpoint),  # None means max, if a temp is required
+                ZONE_MODE_LOOKUP[mode],
+                "FFFFFF" if duration is None else f"{duration:06X}",
+                "" if until is None else dtm_to_hex(until),
+            )
+        )
 
         return cls(W_, "2349", payload, ctl_id, **kwargs)
 
@@ -753,6 +805,37 @@ class Command:
         return cls(I_, "7FFF", payload[:48], NUL_DEV_ADDR.id, **kwargs)
 
 
+# A convenience dict
+_COMMANDS = {
+    f"{I_}/0002": Command.put_outdoor_temp,
+    f"{RQ}/0004": Command.get_zone_name,
+    f"{W_}/0004": Command.set_zone_name,
+    f"{RQ}/000A": Command.get_zone_config,
+    f"{W_}/000A": Command.set_zone_config,
+    f"{RQ}/0100": Command.get_system_language,
+    f"{RQ}/0418": Command.get_system_log_entry,
+    f"{RQ}/1030": Command.get_mix_valve_params,
+    f"{W_}/1030": Command.set_mix_valve_params,
+    f"{RQ}/10A0": Command.get_dhw_params,
+    f"{W_}/10A0": Command.set_dhw_params,
+    f"{RQ}/1100": Command.get_tpi_params,
+    f"{W_}/1100": Command.set_tpi_params,
+    f"{RQ}/1260": Command.get_dhw_temp,
+    f"{RQ}/12B0": Command.get_zone_window_state,
+    f"{RQ}/1F41": Command.get_dhw_mode,
+    f"{W_}/1F41": Command.set_dhw_mode,
+    f"{RQ}/2349": Command.get_zone_mode,
+    f"{W_}/2349": Command.set_zone_mode,
+    f"{RQ}/2E04": Command.get_system_mode,
+    f"{W_}/2E04": Command.set_system_mode,
+    f"{I_}/30C9": Command.put_sensor_temp,
+    f"{RQ}/30C9": Command.get_zone_temperature,
+    f"{RQ}/313F": Command.get_system_time,
+    f"{W_}/313F": Command.set_system_time,
+    f"{RQ}/3220": Command.get_opentherm_data,
+}  # TODO: RQ/0404 (Zone & DHW)
+
+
 class FaultLog:  # 0418  # TODO: used a NamedTuple
     """The fault log of a system."""
 
@@ -1024,7 +1107,7 @@ class Schedule:  # 0404
         frags = [struct.pack("<xxxxBxxxBxxxHxxHxx", *s) for s in frags]
 
         cobj = zlib.compressobj(level=9, wbits=14)
-        blob = b"".join([cobj.compress(s) for s in frags]) + cobj.flush()
+        blob = b"".join(cobj.compress(s) for s in frags) + cobj.flush()
         blob = blob.hex().upper()
 
         return [blob[i : i + 82] for i in range(0, len(blob), 82)]
