@@ -14,13 +14,13 @@ from typing import Any, Optional, Tuple, Union
 
 from . import parsers
 from .const import (
+    _0005_ZONE_TYPE,
     ATTR_DHW_SENSOR,
     ATTR_DHW_VALVE,
     ATTR_DHW_VALVE_HTG,
     ATTR_HTG_CONTROL,
     ATTR_ZONE_ACTUATORS,
     ATTR_ZONE_SENSOR,
-    CODE_0005_ZONE_TYPE,
     DEVICE_TYPES,
     MSG_FORMAT_10,
     MSG_FORMAT_18,
@@ -39,13 +39,7 @@ from .exceptions import (
 )
 from .packet import _PKT_LOGGER  # TODO: bad packets are being logged twice!
 from .ramses import CODES_WITH_COMPLEX_IDX, CODES_WITHOUT_IDX, RAMSES_CODES
-from .schema import (
-    DONT_CREATE_ENTITIES,
-    DONT_UPDATE_ENTITIES,
-    ENABLE_EAVESDROP,
-    REDUCE_PROCESSING,
-    USE_NAMES,
-)
+from .schema import DONT_CREATE_ENTITIES, DONT_UPDATE_ENTITIES
 
 # from .systems import Evohome
 
@@ -56,24 +50,23 @@ I_, RQ, RP, W_ = " I", "RQ", "RP", " W"
 # TODO: WIP
 MSG_TIMEOUTS = {
     "0004": {I_: td(days=1), RP: td(days=1)},
-    "0005": {I_: td(days=1), RP: td(days=1)},
     "000A": {I_: td(days=1), RP: td(days=1)},
-    "000C": {I_: td(days=1), RP: td(days=1)},
     "0100": {I_: td(days=1), RP: td(days=1)},
     "1060": {I_: td(days=1)},
-    "10E0": {I_: td(days=7), RP: td(days=7)},
+    "10A0": {RP: td(hours=4)},
+    "1100": {I_: td(days=1), RP: td(days=1)},
     "1260": {I_: td(hours=1), RP: td(hours=1)},
     "12B0": {I_: td(hours=1), RP: td(hours=1)},
     "1F41": {I_: td(hours=4), RP: td(hours=4)},
-    "2309": {I_: td(minutes=15), RP: td(minutes=4)},
+    "2309": {I_: td(minutes=30), RP: td(minutes=30)},
     "2349": {I_: td(hours=4), RP: td(hours=4)},
     "2E04": {I_: td(hours=4), RP: td(hours=4)},
     "313F": {I_: td(seconds=3), RP: td(seconds=3)},
     "3150": {I_: td(minutes=20)},
     "3C09": {I_: td(hours=4), RP: td(hours=4)},
-}
+}  # not arrays
 
-DEV_MODE = __dev_mode__ and False
+DEV_MODE = True or __dev_mode__ and False
 
 _LOGGER = logging.getLogger(__name__)
 if DEV_MODE:
@@ -137,7 +130,7 @@ class Message:
 
         def display_name(dev: Union[Address, Device]) -> str:
             """Return a formatted device name, uses a friendly name if there is one."""
-            if self._gwy.config[USE_NAMES]:
+            if self._gwy.config.use_names:
                 try:
                     return f"{self._gwy.known_devices[dev.id]['name']:<18}"
                 except (KeyError, TypeError):
@@ -170,7 +163,7 @@ class Message:
 
         payload = self.raw_payload if self.len < 4 else f"{self.raw_payload[:5]}..."[:9]
 
-        _format = MSG_FORMAT_18 if self._gwy.config[USE_NAMES] else MSG_FORMAT_10
+        _format = MSG_FORMAT_18 if self._gwy.config.use_names else MSG_FORMAT_10
         self._str = _format.format(
             src, dst, self.verb, self.code_name, payload, self._payload
         )
@@ -234,7 +227,7 @@ class Message:
             self._haz_simple_idx = False  # has no idx, even though some != "00"
         elif RAMSES_CODES.get(self.code):
             self._haz_simple_idx = (
-                not RAMSES_CODES[self.code].get(self.verb, "")[:3] == "^00"
+                RAMSES_CODES[self.code].get(self.verb, "")[:3] != "^00"
             )  # has no _idx
             assert (
                 self._haz_simple_idx or self.raw_payload[:2] == "00"
@@ -276,14 +269,14 @@ class Message:
             # grep -E ' (I|RP).* 1FC9 '  #  from 01:/13:/other (not W)
             self._is_array = True
 
-        # elif self.verb == RP:
-        #     self._is_array = False
+        elif self.verb == RP:
+            self._is_array = False
 
-        # # 087  I 092 --:------ --:------ 12:126457 2309 006 0107D0-020708
-        # # 090  I 093 --:------ --:------ 12:126457 30C9 003 017FFF
-        # # 089  I 094 --:------ --:------ 12:126457 000A 012 010001F40BB8-020001F40BB8
-        # elif self.code in ("000A", "2309", "30C9") and self.src.type == "12":
-        #     self._is_array = self.verb == I_ and self.dst.id == "--:------"
+        # 087  I 092 --:------ --:------ 12:126457 2309 006 0107D0-020708
+        # 090  I 093 --:------ --:------ 12:126457 30C9 003 017FFF
+        # 089  I 094 --:------ --:------ 12:126457 000A 012 010001F40BB8-020001F40BB8
+        elif self.code in ("000A", "2309", "30C9") and self.src.type == "12":
+            self._is_array = self.verb == I_ and self.dst.id == "--:------"
 
         elif self.verb not in (I_, RP) or self.src.id != self.dst.id:
             self._is_array = False
@@ -340,6 +333,12 @@ class Message:
             if self.verb in (RQ, W_):
                 timeout = td(seconds=3)
 
+            elif self.code in ("0005", "000C", "10E0"):
+                return  # TODO: exclude/remove devices caused by corrupt ADDRs?
+
+            elif self.code == "1FC9" and self.verb == RP:
+                return  # TODO: check other verbs, they seem variable
+
             elif self.code == "1F09":
                 timeout = td(seconds=self.payload["remaining_seconds"])
 
@@ -357,30 +356,47 @@ class Message:
 
             return timeout or td(minutes=60)
 
-        if self._is_expired == self.HAS_EXPIRED:
+        if self._is_expired == self.HAS_EXPIRED:  # TODO: or CANT_EXPIRE
             return self._is_expired
 
-        timeout = _timeout()
+        timeout = _timeout()  # TODO: needs fixing
 
         if timeout is None:  # treat as never expiring
-            self._is_expired = self.NOT_EXPIRED
-            _logger_send(_LOGGER.debug, "msg is not expirable ({dtm_now}, {timeout})")
+            self._is_expired = self.NOT_EXPIRED  # TODO: CANT_EXPIRE
+            # _logger_send(
+            #    _LOGGER.debug,
+            #    "msg is not expirable",
+            # )
             return self._is_expired
 
-        if self._gwy.serial_port:
-            dtm_now = dt.now()
-        else:
-            dtm_now = self._gwy._prev_msg.dtm if self._gwy._prev_msg else self.dtm
+        dtm_now = (
+            dt.now()
+            if self._gwy.serial_port
+            else (self._gwy._prev_msg.dtm if self._gwy._prev_msg else self.dtm)
+        )
 
         if self.dtm < dtm_now - timeout * 2:
+            if self._is_expired != self.HAS_EXPIRED:
+                _logger_send(
+                    _LOGGER.error,
+                    f"msg has tombstoned ({dtm_now - self.dtm}, {timeout})",
+                )
             self._is_expired = self.HAS_EXPIRED
-            _logger_send(_LOGGER.error, f"msg has tombstoned ({dtm_now}, {timeout})")
+
         elif self.dtm < dtm_now - timeout * 1:
+            if self._is_expired != self.IS_EXPIRING:
+                _logger_send(
+                    _LOGGER.info,
+                    f"msg has expired ({dtm_now - self.dtm}, {timeout})",
+                )
             self._is_expired = self.IS_EXPIRING
-            _logger_send(_LOGGER.info, f"msg has expired ({dtm_now}, {timeout})")
+
         else:
             self._is_expired = self.NOT_EXPIRED
-            _logger_send(_LOGGER.debug, f"msg has not expired ({dtm_now}, {timeout})")
+            # _logger_send(
+            #     _LOGGER.debug,
+            #     f"msg has not expired ({dtm_now - self.dtm}, {timeout})",
+            # )
         return self._is_expired
 
     @property
@@ -497,12 +513,12 @@ def process_msg(msg: Message) -> None:
 
         if this.src.type in ("01", "23") and this.src is not this.dst:  # TODO: all CTLs
             this.src = this._gwy._get_device(this.src, ctl_addr=this.src)
-            ctl_addr = this.src if msg._gwy.config[ENABLE_EAVESDROP] else None
+            ctl_addr = this.src if msg._gwy.config.enable_eavesdrop else None
             this._gwy._get_device(this.dst, ctl_addr=ctl_addr)
 
         elif this.dst.type in ("01", "23") and this.src is not this.dst:  # all CTLs
             this.dst = this._gwy._get_device(this.dst, ctl_addr=this.dst)
-            ctl_addr = this.dst if msg._gwy.config[ENABLE_EAVESDROP] else None
+            ctl_addr = this.dst if msg._gwy.config.enable_eavesdrop else None
             this._gwy._get_device(this.src, ctl_addr=ctl_addr)
 
         # TODO: will need other changes before these two will work...
@@ -526,6 +542,7 @@ def process_msg(msg: Message) -> None:
             this._gwy._get_device(this.src, ctl_addr=this.dst)
 
         else:
+            # beware:  I --- --:------ --:------ 10:078099 1FD4 003 00F079
             [this._gwy._get_device(d) for d in (this.src, this.dst)]
 
         # where possible, swap each Address for its corresponding Device
@@ -543,7 +560,7 @@ def process_msg(msg: Message) -> None:
 
         # TODO: a I/0005: zones have changed & may need a restart (del) or not (add)
         if this.code == "0005":  # RP, and also I
-            if this._payload["zone_type"] in CODE_0005_ZONE_TYPE.values():
+            if this._payload["zone_type"] in _0005_ZONE_TYPE.values():
                 [
                     evo._get_zone(
                         f"{idx:02X}",
@@ -554,7 +571,7 @@ def process_msg(msg: Message) -> None:
                 ]
 
         if this.code == "000C" and this.src.type == "01":
-            if this.payload["devices"]:
+            if this.payload["devices"]:  # i.e. if len(devices) > 0
                 devices = [this.src.device_by_id[d] for d in this.payload["devices"]]
 
                 if this.payload["device_class"] == ATTR_ZONE_SENSOR:
@@ -565,24 +582,22 @@ def process_msg(msg: Message) -> None:
                         pass
 
                 elif this.payload["device_class"] == ATTR_ZONE_ACTUATORS:
-                    # TODO: is this better, or...
                     # evo._get_zone(this.payload["zone_idx"], actuators=devices)
-                    # TODO: is it this one?
+                    # TODO: whihc is better, above or below?
                     zone = evo._get_zone(this.payload["zone_idx"])
-                    for d in devices:
-                        d._set_zone(zone)
+                    [d._set_parent(zone) for d in devices]
 
                 elif this.payload["device_class"] == ATTR_HTG_CONTROL:
                     evo._set_htg_control(devices[0])
 
                 elif this.payload["device_class"] == ATTR_DHW_SENSOR:
-                    evo._get_zone("HW")._set_sensor(devices[0])
+                    evo._get_dhw()._set_sensor(devices[0])
 
                 elif this.payload["device_class"] == ATTR_DHW_VALVE:
-                    evo._get_zone("HW")._set_dhw_valve(devices[0])
+                    evo._get_dhw()._set_dhw_valve(devices[0])
 
                 elif this.payload["device_class"] == ATTR_DHW_VALVE_HTG:
-                    evo._get_zone("HW")._set_htg_valve(devices[0])
+                    evo._get_dhw()._set_htg_valve(devices[0])
 
             elif this.payload["device_class"] == ATTR_HTG_CONTROL:
                 # TODO: maybe the htg controller is an OTB? via eavesdropping
@@ -593,9 +608,9 @@ def process_msg(msg: Message) -> None:
         # # TODO: needs work, e.g. RP/1F41 (excl. null_rp)
         # elif this.code in ("10A0", "1F41"):
         #     if isinstance(this.dst, Device) and this.dst._is_controller:
-        #         this.dst._get_zone("HW")
+        #         this.dst._get_dhw()
         #     else:
-        #         evo._get_zone("HW ")
+        #         evo._get_dhw()
 
         # # TODO: also process ufh_idx (but never domain_id)
         # elif isinstance(this._payload, dict):
@@ -640,16 +655,17 @@ def process_msg(msg: Message) -> None:
         # if not this.payload:
         #     return
 
+        # # TODO: what follows is a mess - needs sorting out
+        # evo = this._gwy.system_by_id.get(this.src.id)
+        # if evo:
+        #     evo._handle_msg(this)
+
         for evo in this._gwy.systems:
             # if this.src == evo:  # TODO: or this.src.id == evo.id?
             if this.code in ("10A0", "1260", "1F41") and evo._dhw is not None:
                 evo._dhw._handle_msg(this)
             break
 
-        #     if this.src.controller == evo:  # TODO: this.src.controller.id == evo.id?
-        #         evo._handle_msg(this, prev)  # TODO: WIP
-        #         break
-
         evo = this.src._evo if hasattr(this.src, "_evo") else None
         if evo is None:
             evo = this.dst._evo if hasattr(this.dst, "_evo") else None
@@ -684,7 +700,7 @@ def process_msg(msg: Message) -> None:
     if msg.src.type == "18":
         return
 
-    if msg._gwy.config[REDUCE_PROCESSING] >= DONT_CREATE_ENTITIES:
+    if msg._gwy.config.reduce_processing >= DONT_CREATE_ENTITIES:
         return
 
     try:  # process the payload
@@ -692,7 +708,7 @@ def process_msg(msg: Message) -> None:
         # if msg._evo:  # TODO:
         create_zones(msg)  # create zones & (TBD) ufh_zones too?
 
-        if msg._gwy.config[REDUCE_PROCESSING] < DONT_UPDATE_ENTITIES:
+        if msg._gwy.config.reduce_processing < DONT_UPDATE_ENTITIES:
             update_entities(msg, msg._gwy._prev_msg)  # update the state database
 
     except (AssertionError, NotImplementedError) as err:
