@@ -3,6 +3,11 @@
 #
 """RAMSES RF - payload processors."""
 
+# Many thanks to:
+# - Evsdd: 0404
+# - Ierlandfan: 3150, 31D9, 31DA, others
+# - ReneKlootwijk: 3EF0
+
 import logging
 import re
 from datetime import datetime as dt
@@ -12,11 +17,11 @@ from typing import Optional, Union
 
 from .command import Command
 from .const import (
+    _000C_DEVICE_TYPE,
+    _0005_ZONE_TYPE,
     ATTR_DHW_VALVE,
     ATTR_DHW_VALVE_HTG,
     ATTR_HTG_CONTROL,
-    CODE_000C_DEVICE_TYPE,
-    CODE_0005_ZONE_TYPE,
     CODE_0418_DEVICE_CLASS,
     CODE_0418_FAULT_STATE,
     CODE_0418_FAULT_TYPE,
@@ -48,7 +53,6 @@ from .opentherm import (
     parity,
 )
 from .ramses import RAMSES_CODES, RAMSES_DEVICES, RQ_MAY_HAVE_PAYLOAD
-from .schema import MAX_ZONES
 
 IDX_NAMES = {
     "0002": "other_idx",  # non-evohome: hometronics
@@ -100,6 +104,16 @@ def _idx_idx(seqx, code, src_type, dst_type) -> dict:
             )
             return {idx_name: seqx}
 
+        if src_type != dst_type and not {src_type} & {
+            "01",
+            "12",
+            "23",
+        }:  # to/from a controller
+            idx_name = (
+                "zone_idx" if src_type in ("01", "02", "23", "18") else "parent_idx"
+            )
+            return {idx_name: seqx}
+
         #  I --- 02:001075 --:------ 02:001075 3150 006 000001000200
         #  I --- 02:000921 --:------ 02:000921 3150 006 00B201C802A8
         if src_type == "02":  # and src_type == dst_type:  # src == dst:
@@ -149,7 +163,7 @@ def _idx(seqx, msg) -> dict:
 
     if "zone_idx" in result or "parent_idx" in result:
         assert (
-            int(msg.raw_payload[:2], 16) < msg._gwy.config[MAX_ZONES]
+            int(msg.raw_payload[:2], 16) < msg._gwy.config.max_zones
         ), f"invalid zone_idx: '{seqx}'"
     elif "domain_id" in result:
         assert seqx in ("F9", "FA", "FC"), f"invalid domain_id: '{seqx}'"
@@ -161,7 +175,7 @@ def _idx(seqx, msg) -> dict:
         assert seqx in ("00", "01", "21"), f"invalid hvac_id: '{seqx}'"
     elif "other_idx" in result:
         assert (
-            int(msg.raw_payload[4:6], 16) < msg._gwy.config[MAX_ZONES]
+            int(msg.raw_payload[4:6], 16) < msg._gwy.config.max_zones
         ), f"unknown other_idx: '{seqx}'"
 
     elif result != {}:
@@ -406,7 +420,7 @@ def parser_0001(payload, msg) -> Optional[dict]:
     assert msg.verb in (I_, W_), msg.verb
     assert msg.len == 5, msg.len
     assert payload[:2] in ("FC", "FF") or (
-        int(payload[:2], 16) < msg._gwy.config[MAX_ZONES]
+        int(payload[:2], 16) < msg._gwy.config.max_zones
     ), payload[:2]
     assert payload[2:6] in ("0000", "FFFF"), payload[2:6]
     assert payload[6:8] in ("02", "05"), payload[6:8]
@@ -465,18 +479,18 @@ def parser_0005(payload, msg) -> Optional[dict]:
         assert len(seqx) in (8, 12)  # 8 for evohome, 12 for Hometronics (16 zones)
         assert seqx[:2] == payload[:2]
         assert seqx[:2] == "00"  # done in _idx
-        # assert payload[2:4] in CODE_0005_ZONE_TYPE, f"Unknown zone_type: {seqx[2:4]}"
+        # assert payload[2:4] in _0005_ZONE_TYPE, f"Unknown zone_type: {seqx[2:4]}"
 
-        max_zones = msg._gwy.config[MAX_ZONES]
+        max_zones = msg._gwy.config.max_zones
         return {
             "zone_mask": (_flag8(seqx[4:6]) + _flag8(seqx[6:8]))[:max_zones],
-            "zone_type": CODE_0005_ZONE_TYPE.get(seqx[2:4], seqx[2:4]),
+            "zone_type": _0005_ZONE_TYPE.get(seqx[2:4], seqx[2:4]),
         }
 
     if msg.verb == RQ:
         assert payload[:2] == "00", payload[:2]
         return {
-            "zone_type": CODE_0005_ZONE_TYPE.get(payload[2:4], payload[2:4]),
+            "zone_type": _0005_ZONE_TYPE.get(payload[2:4], payload[2:4]),
         }
 
     assert msg.verb in (I_, RP)
@@ -526,7 +540,7 @@ def parser_0008(payload, msg) -> Optional[dict]:
         if msg.verb == I_ and msg.src.type in ("01", "02") and msg.src is msg.dst:
             if payload[:2] in ("F9", "FA", "FC"):
                 return {"domain_id": payload[:2]}
-            assert int(payload[:2], 16) < msg._gwy.config[MAX_ZONES]
+            assert int(payload[:2], 16) < msg._gwy.config.max_zones
             return {"zone_idx": payload[:2]}
         assert payload[:2] == "00"
         return {}
@@ -561,9 +575,7 @@ def parser_0009(payload, msg) -> Union[dict, list]:
     # 095  I --- 23:100224 --:------ 23:100224 0009 003 0100FF  # 2-zone ST9520C
 
     def _parser(seqx) -> dict:
-        assert (
-            seqx[:2] in ("F9", "FC") or int(seqx[:2], 16) < msg._gwy.config[MAX_ZONES]
-        )
+        assert seqx[:2] in ("F9", "FC") or int(seqx[:2], 16) < msg._gwy.config.max_zones
         assert seqx[2:4] in ("00", "01"), seqx[2:4]
         assert seqx[4:] in ("00", "FF"), seqx[4:]
 
@@ -657,15 +669,15 @@ def parser_000c(payload, msg) -> Optional[dict]:
             return {"domain_id": "FC"}
 
         assert (
-            int(seqx, 16) < msg._gwy.config[MAX_ZONES]
+            int(seqx, 16) < msg._gwy.config.max_zones
         ), f"invalid zone_idx: '{seqx}' (0x03)"
         return {"zone_idx": seqx}
 
     def _parser(seqx) -> dict:
         # assert len(seqx) == 12, len(seqx)  # not needed
         assert seqx[:2] == payload[:2], seqx[:2]
-        # assert seqx[2:4] in CODE_000C_DEVICE_TYPE, f"Unknown device_type: {seqx[2:4]}"
-        assert seqx[4:6] == "7F" or int(seqx[4:6], 16) < msg._gwy.config[MAX_ZONES]
+        # assert seqx[2:4] in _000C_DEVICE_TYPE, f"Unknown device_type: {seqx[2:4]}"
+        assert seqx[4:6] == "7F" or int(seqx[4:6], 16) < msg._gwy.config.max_zones
         return {hex_id_to_dec(seqx[6:12]): seqx[4:6]}
 
     if msg.verb == RQ:
@@ -673,7 +685,7 @@ def parser_000c(payload, msg) -> Optional[dict]:
     else:
         assert msg.len >= 6 and msg.len % 6 == 0, msg.len  # assuming not RQ
 
-    device_class = CODE_000C_DEVICE_TYPE.get(payload[2:4], f"unknown_{payload[2:4]}")
+    device_class = _000C_DEVICE_TYPE.get(payload[2:4], f"unknown_{payload[2:4]}")
     if device_class == ATTR_DHW_VALVE and msg.raw_payload[:2] == "01":
         device_class = ATTR_DHW_VALVE_HTG
 
@@ -815,7 +827,7 @@ def parser_0418(payload, msg) -> Optional[dict]:
     assert int(payload[4:6], 16) < 64, payload[4:6]
     assert payload[8:10] in CODE_0418_FAULT_TYPE, payload[8:10]
 
-    assert int(payload[10:12], 16) < msg._gwy.config[MAX_ZONES] or (
+    assert int(payload[10:12], 16) < msg._gwy.config.max_zones or (
         payload[10:12] in ("F9", "FA", "FC")  # "1C"?
     ), f"unexpected domain_id: {payload[10:12]}"
     assert payload[12:14] in CODE_0418_DEVICE_CLASS, payload[12:14]
@@ -835,7 +847,7 @@ def parser_0418(payload, msg) -> Optional[dict]:
     if payload[12:14] != "00":  # Controller
         key_name = (
             "zone_id"
-            if int(payload[10:12], 16) < msg._gwy.config[MAX_ZONES]
+            if int(payload[10:12], 16) < msg._gwy.config.max_zones
             else "domain_id"
         )  # TODO: don't use zone_idx (for now)
         result.update({key_name: payload[10:12]})
@@ -973,6 +985,11 @@ def parser_1090(payload, msg) -> dict:
 
 @parser_decorator  # dhw_params
 def parser_10a0(payload, msg) -> Optional[dict]:
+    # These pairs occur every 4h...
+    # RQ --- 07:045960 01:145038 --:------ 10A0 006 00-1087-00-03E4
+    # RP --- 01:145038 07:045960 --:------ 10A0 006 00-109A-00-03E8
+
+    # these may not be reliable...
     # RQ --- 01:136410 10:067219 --:------ 10A0 002 0000
     # RQ --- 07:017494 01:078710 --:------ 10A0 006 00-1566-00-03E4
 
@@ -982,6 +999,7 @@ def parser_10a0(payload, msg) -> Optional[dict]:
     # RQ --- 07:030741 01:102458 --:------ 10A0 006 00-181F-00-03E4
     # RQ --- 07:036831 23:100224 --:------ 10A0 006 01-1566-00-03E4 (non-evohome)
 
+    # these from a RFG...
     # RQ --- 30:185469 01:037519 --:------ 0005 002 000E
     # RP --- 01:037519 30:185469 --:------ 0005 004 000E0300  # two DHW valves
     # RQ --- 30:185469 01:037519 --:------ 10A0 001 01 (01 )
@@ -1072,8 +1090,8 @@ def parser_10e0(payload, msg) -> Optional[dict]:
 
     return {  # TODO: add version?
         "unknown": payload[2:20],
-        "date_2": date_2 if date_2 else "0000-00-00",
-        "date_1": date_1 if date_1 else "0000-00-00",
+        "date_2": date_2 or "0000-00-00",
+        "date_1": date_1 or "0000-00-00",
         "description": description,
         "_unknown_2": payload[38 + len(description) * 2 :],
     }
@@ -1178,6 +1196,28 @@ def parser_12a0(payload, msg) -> Optional[dict]:
     # assert msg.len == 6 if type == ?? else 2, msg.len
     assert payload[:2] == "00", payload[:2]  # domain?
 
+    RHUM_STATE = {
+        "EF": "not available",
+        "F0": "sensor short circuit",
+        "F1": "sensor open",
+        "F2": "not available",
+        "F3": "sensor value too high",
+        "F4": "sensor value too low",
+        "F5": "sensor unreliable",
+    }  # EFFF = N/A
+    assert payload[2:4] in RHUM_STATE or int(payload[2:4], 16) <= 100
+
+    # TEMP_STATE = {
+    #     "7F": "not available",
+    #     "80": "sensor short circuit",
+    #     "81": "sensor open",
+    #     "82": "not available",
+    #     "83": "sensor value too high",
+    #     "84": "sensor value too low",
+    #     "85": "sensor unreliable",
+    # }  # 7F = N/A, same for 1298
+    # assert payload[4:6] in TEMP_STATE or ...
+
     rh = int(payload[2:4], 16) / 100 if payload[2:4] != "EF" else None
     if msg.len == 2:
         return {"relative_humidity": rh}
@@ -1305,7 +1345,7 @@ def parser_1fc9(payload, msg) -> Optional[dict]:
             "FC",
             "FF",
         ):  # or: not in DOMAIN_TYPE_MAP: ??
-            assert int(seqx[:2], 16) < msg._gwy.config[MAX_ZONES]
+            assert int(seqx[:2], 16) < msg._gwy.config.max_zones
         return [seqx[:2], seqx[2:6], hex_id_to_dec(seqx[6:])]
 
     assert msg.len >= 6 and msg.len % 6 == 0, msg.len  # assuming not RQ
@@ -1400,7 +1440,7 @@ def parser_22f1(payload, msg) -> Optional[dict]:
 
     if bitmap in FanSwitch.FAN_MODES:
         _action = {FanSwitch.FAN_MODE: FanSwitch.FAN_MODES[bitmap]}
-    elif bitmap in (9, 10):
+    elif bitmap in {9, 10}:
         _action = {FanSwitch.HEATER_MODE: FanSwitch.HEATER_MODES[bitmap]}
     else:
         _action = {}
@@ -1498,7 +1538,7 @@ def parser_2349(payload, msg) -> Optional[dict]:
 @parser_decorator  # hometronics _state (of unknwon)
 def parser_2d49(payload, msg) -> dict:
     assert (
-        payload[:2] in ("88", "FD") or int(payload[:2], 16) < msg._gwy.config[MAX_ZONES]
+        payload[:2] in ("88", "FD") or int(payload[:2], 16) < msg._gwy.config.max_zones
     ), payload[:2]
     assert payload[2:] in ("0000", "C800"), payload[2:]  # would "FFFF" mean N/A?
     # assert msg.len == 3, msg.len  # implied
@@ -1627,10 +1667,18 @@ def parser_3150(payload, msg) -> Optional[dict]:
     # event-driven, and periodically; FC domain is maximum of all zones
     # TODO: all have a valid domain will UFC/CTL respond to an RQ, for FC, for a zone?
 
-    #  I --- 04:136513 --:------ 01:158182 3150 002 01CA < Often see CA
+    #  I --- 04:136513 --:------ 01:158182 3150 002 01CA < often see CA
+
+    # VALVE_STATE = {
+    #     0x00: "open circuit",
+    #     0x01: "short circuit",
+    #     0x0D: "damper/valve stuck",
+    #     0x0E: "actuator stuck",
+    # }  # VALVE_STATE.get(seqx[2:], "malfunction")
 
     def _parser(seqx) -> dict:
         # assert seqx[:2] == "FC" or (int(seqx[:2], 16) < MAX_ZONES)  # <5, 8 for UFC
+        assert int(seqx[2:], 16) & 0xF0 == 0xF0 or int(seqx[2:], 16) <= 200
         return {**_idx(seqx[:2], msg), "heat_demand": _percent(seqx[2:])}
 
     if msg.src.type == "02" and msg.is_array:  # TODO: hometronics only?
@@ -1642,12 +1690,20 @@ def parser_3150(payload, msg) -> Optional[dict]:
 
 @parser_decorator  # ???
 def parser_31d9(payload, msg) -> Optional[dict]:
+    assert payload[:2] in ("00", "01", "21"), payload[2:4]
     assert payload[2:4] in ("00", "06", "80"), payload[2:4]
     assert payload[4:6] == "FF" or int(payload[4:6], 16) <= 200, payload[4:6]
 
+    bitmap = int(payload[2:4], 16)
+
     result = {
-        FanSwitch.FAN_RATE: _percent(payload[4:6]),  # NOTE: is 31DA/payload[38:40]
-        "_unknown_0": payload[2:4],
+        "exhaust_fan_speed": _percent(payload[4:6]),  # NOTE: is 31DA/payload[38:40]
+        "_passive": bool(bitmap & 0x02),
+        "_damper_only": bool(bitmap & 0x04),
+        "_filter_dirty": bool(bitmap & 0x20),
+        "_frost_cycle": bool(bitmap & 0x40),
+        "_has_fault": bool(bitmap & 0x80),
+        "_bitmap_0": payload[2:4],
     }
 
     if msg.len == 3:  # usu: I -->20: (no seq#)
@@ -1669,7 +1725,7 @@ def parser_31d9(payload, msg) -> Optional[dict]:
 @parser_decorator  # UFC HCE80 (Nuaire humidity)
 def parser_31da(payload, msg) -> Optional[dict]:
 
-    FAN_INFO = {
+    CODE_31DA_FAN_INFO = {
         0x00: "off",
         0x01: "speed 1",
         0x02: "speed 2",
@@ -1707,7 +1763,8 @@ def parser_31da(payload, msg) -> Optional[dict]:
             raise ValueError(result)
         return result / 100 / precision
 
-    assert msg.len == 29, f"expected length 29, not {msg.len}"
+    # I --- 37:261128 --:------ 37:261128 31DA 029 00004007D045EF7FFF7FFF7FFF7FFFF808EF03C8000000EFEF7FFF7FFF
+    # I --- 37:053679 --:------ 37:053679 31DA 030 00EF007FFF41EF7FFF7FFF7FFF7FFFF800EF0134000000EFEF7FFF7FFF00
 
     assert payload[2:4] in ("00", "EF"), payload[2:4]
     assert payload[4:6] in ("00", "40"), payload[4:6]
@@ -1731,7 +1788,7 @@ def parser_31da(payload, msg) -> Optional[dict]:
     assert payload[46:48] in ("00", "EF"), payload[46:48]
     assert payload[48:50] == "EF", payload[48:50]
     assert payload[50:54] == "7FFF", payload[50:54]
-    assert payload[54:58] == "7FFF", payload[54:58]
+    assert payload[54:58] == "7FFF", payload[54:58]  # or: FFFF?
 
     return {
         "air_quality": _percent(payload[2:4]),
@@ -1745,7 +1802,7 @@ def parser_31da(payload, msg) -> Optional[dict]:
         "outdoor_temperature": _double(payload[26:30], factor=100),  # TODO: 1290?
         "speed_cap": int(payload[30:34], 16),
         "bypass_pos": _percent(payload[34:36]),
-        "fan_info": FAN_INFO[int(payload[36:38], 16) & 0x1F],
+        "fan_info": CODE_31DA_FAN_INFO[int(payload[36:38], 16) & 0x1F],
         "exhaust_fan_speed": _percent(payload[38:40]),  # NOTE: 31D9/payload[4:6]
         "supply_fan_speed": _percent(payload[40:42]),
         "remaining_time": _double(payload[42:46]),  # mins NOTE: 22F3/payload[2:6]
@@ -1866,7 +1923,7 @@ def parser_3b00(payload, msg) -> Optional[dict]:
         return {}
 
     assert msg.len == 2, msg.len
-    assert payload[:2] in {"01": "FC", "13": "00", "23": "FC"}.get(msg.src.type, "00")
+    assert payload[:2] == {"01": "FC", "13": "00", "23": "FC"}.get(msg.src.type, "00")
     assert payload[2:] == "C8", payload[2:]  # Could it be a percentage?
 
     return {
@@ -1877,7 +1934,6 @@ def parser_3b00(payload, msg) -> Optional[dict]:
 
 @parser_decorator  # actuator_state
 def parser_3ef0(payload, msg) -> dict:
-    # Some of this data thanks to @ReneKlootwijk
 
     if msg.src.type in "08":  # Honeywell Jasper ?HVAC
         assert msg.len == 20, f"expecting len 20, got: {msg.len}"
