@@ -16,7 +16,7 @@ from typing import Callable, List, Optional, Tuple
 from .command import ARGS, DEAMON, EXPIRES, FUNC, TIMEOUT, Command
 from .const import __dev_mode__
 from .message import Message
-from .schema import DISABLE_SENDING, DONT_CREATE_MESSAGES, REDUCE_PROCESSING
+from .schema import DONT_CREATE_MESSAGES
 
 DEV_MODE = __dev_mode__ and False
 
@@ -151,19 +151,19 @@ class MessageTransport(asyncio.Transport):
                 # see  also: PktProtocolQos.send_data()
                 _LOGGER.error("MsgTransport._pkt_receiver(%s): Expired callback", hdr)
                 callback[FUNC](False, *callback.get(ARGS, tuple()))
-                callback["expired"] = not callback.get(DEAMON, False)
+                callback["expired"] = not callback.get(DEAMON, False)  # HACK:
 
         self._callbacks = {  # 2nd, discard any expired callbacks
             hdr: callback
             for hdr, callback in self._callbacks.items()
             if callback.get(DEAMON)
-            or (callback[EXPIRES] >= pkt._dtm and not not callback.get("expired"))
+            or (callback[EXPIRES] >= pkt._dtm and not callback.get("expired"))
         }
 
         if len(self._protocols) == 0:
             return
 
-        if self._gwy.config[REDUCE_PROCESSING] >= DONT_CREATE_MESSAGES:
+        if self._gwy.config.reduce_processing >= DONT_CREATE_MESSAGES:
             return
 
         try:
@@ -329,7 +329,7 @@ class MessageTransport(asyncio.Transport):
         if self._is_closing:
             raise RuntimeError("MsgTransport is closing or has closed")
 
-        if self._gwy.config[DISABLE_SENDING]:
+        if self._gwy.config.disable_sending:
             message = "MsgTransport.write(%s): sending disabled: discarded"
             if DEV_MODE:
                 _LOGGER.warning(message, cmd)
@@ -342,7 +342,7 @@ class MessageTransport(asyncio.Transport):
 
             self._que.put_nowait(cmd)  # was: self._que.put_nowait(cmd)
 
-        self.get_write_buffer_size()
+        # self.get_write_buffer_size()  # TODO: how to auto unpause?
 
     def writelines(self, list_of_cmds):
         """Write a list (or any iterable) of data bytes to the transport.
