@@ -5,6 +5,7 @@
 
 import logging
 import re
+from types import SimpleNamespace
 from typing import Any, Optional, Tuple
 
 import voluptuous as vol
@@ -38,6 +39,7 @@ from .const import (
     __dev_mode__,
 )
 from .const import id_to_address as addr
+from .devices import DEVICE_BY_CLASS_ID
 
 # schema attrs
 ATTR_DEVICE_ID = "device_id"
@@ -151,7 +153,11 @@ FILTER_SCHEMA = vol.Any([], vol.All(vol.Schema([DEVICE_ID]), vol.Length(min=0)))
 KNOWNS_SCHEMA = vol.Schema(
     {
         vol.Optional(DEVICE_ID): vol.Any(
-            None, {vol.Optional("name", default=None): vol.Any(None, str)}
+            None,
+            {
+                vol.Optional("name", default=None): vol.Any(None, str),
+                vol.Optional("dev_class"): vol.Any(*DEVICE_BY_CLASS_ID.keys()),
+            },  # TODO: override type-by-addr.id with dev_class
         )
     },
     extra=vol.PREVENT_EXTRA,
@@ -163,12 +169,12 @@ SYSTEM_PROFILES = (SystemType.EVOHOME, SystemType.HOMETRONICS, SystemType.SUNDIA
 
 HTG_SCHEMA = vol.Schema(
     {
-        vol.Optional(ATTR_HTG_CONTROL, default=None): vol.Any(None, HTG_DEVICE_ID),
+        vol.Required(ATTR_HTG_CONTROL, default=None): vol.Any(None, HTG_DEVICE_ID),
         vol.Optional(ATTR_SYS_PROFILE, default=SystemType.EVOHOME): vol.Any(
             *SYSTEM_PROFILES
         ),
     },
-    extra=vol.ALLOW_EXTRA,  # TODO: remove me
+    # extra=vol.ALLOW_EXTRA,  # TODO: remove me
 )
 DHW_SCHEMA = vol.Schema(
     {
@@ -200,6 +206,7 @@ SENSOR_SCHEMA = vol.Schema(
         # vol.Optional("native_id"): DEVICE_ID,
     }
 )
+# SENSOR_SCHEMA = vol.Any(None, SENSOR_ID, SENSOR_SCHEMA)
 ZONE_SCHEMA = vol.Schema(
     {
         vol.Optional(ATTR_ZONE_TYPE, default=None): vol.Any(None, ZONE_TYPE_SLUGS),
@@ -220,7 +227,7 @@ SYSTEM_SCHEMA = vol.Schema(
         vol.Optional(ATTR_ORPHANS, default=[]): vol.Any([], [DEVICE_ID]),
         vol.Optional(ATTR_ZONES, default={}): vol.Any({}, ZONE_SCHEMA),
     },
-    extra=vol.ALLOW_EXTRA,  # TODO: remove me
+    extra=vol.ALLOW_EXTRA,  # TODO: remove me - But: Causes an issue?
 )
 SYSTEM_SCHEMA = vol.Schema(vol.Any({}, SYSTEM_SCHEMA))
 
@@ -241,16 +248,21 @@ GLOBAL_CONFIG_SCHEMA = vol.Schema(
 )
 
 
-DEV_MODE = __dev_mode__
+DEV_MODE = __dev_mode__  # or True
 
 _LOGGER = logging.getLogger(__name__)
 if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-def load_config_schema(serial_port, input_file, **kwargs) -> Tuple[dict, list, list]:
+def load_config_schema(
+    serial_port, input_file, **kwargs
+) -> Tuple[SimpleNamespace, dict, list, list]:
     """Process the configuration, including any filter lists."""
 
+    schema = {
+        k: v for k, v in kwargs.items() if k not in (ALLOW_LIST, BLOCK_LIST, CONFIG)
+    }
     kwargs = GLOBAL_CONFIG_SCHEMA(kwargs)
 
     allow_list = kwargs.pop(ALLOW_LIST)
@@ -280,6 +292,12 @@ def load_config_schema(serial_port, input_file, **kwargs) -> Tuple[dict, list, l
     if config[DISABLE_SENDING]:
         config[DISABLE_DISCOVERY] = True
 
+    if config[ENABLE_EAVESDROP]:
+        _LOGGER.warning(
+            f"{ENABLE_EAVESDROP} was enabled: not doing so is strongly recommended"
+            " (there be dragons here)"
+        )
+
     if config[ENFORCE_ALLOWLIST] is None:
         config[ENFORCE_ALLOWLIST] = (
             False if config[ENFORCE_BLOCKLIST] else bool(allow_list)
@@ -316,7 +334,7 @@ def load_config_schema(serial_port, input_file, **kwargs) -> Tuple[dict, list, l
     elif config[ENFORCE_BLOCKLIST]:
         _LOGGER.debug(f"A {BLOCK_LIST} has been created, length = {len(block_list)}")
 
-    # TODO: use dict or list?
+    # TODO: use dict or list? - will need to add 18? when we see it?
     # allow_list = (
     #     list(allow_list.keys()) if config[ENFORCE_ALLOWLIST] else []
     # )
@@ -324,29 +342,31 @@ def load_config_schema(serial_port, input_file, **kwargs) -> Tuple[dict, list, l
     #     list(block_list.keys()) if config[ENFORCE_BLOCKLIST] else []
     # )
 
-    return (config, allow_list, block_list)
+    config = SimpleNamespace(**config)
+
+    return (config, schema, allow_list, block_list)
 
 
 def _get_device(gwy, dev_addr, ctl_addr=None, **kwargs) -> Optional[Any]:
     """A wrapper to enforce device filters."""
     err_msg = None
 
-    if gwy.config[ENFORCE_ALLOWLIST]:
+    if gwy.config.enforce_allow_list:
         if ctl_addr and ctl_addr.id not in gwy._include:
-            err_msg = f"{ctl_addr.id} is in the schema, but is not in the allow list"
+            err_msg = f"{ctl_addr.id} is in the {SCHEMA}, but not in the {ALLOW_LIST}"
         elif dev_addr.id not in gwy._include:
-            err_msg = f"{dev_addr.id} is in the schema, but is not in the allow list"
+            err_msg = f"{dev_addr.id} is in the {SCHEMA}, but not in the {ALLOW_LIST}"
 
-    elif gwy.config[ENFORCE_BLOCKLIST]:
+    elif gwy.config.enforce_block_list:
         if ctl_addr and ctl_addr.id in gwy._exclude:
-            err_msg = f"{ctl_addr.id} is in the schema, but is also in the block list"
+            err_msg = f"{ctl_addr.id} is in the {SCHEMA}, but also in the {BLOCK_LIST}"
         elif dev_addr.id not in gwy._include:
-            err_msg = f"{dev_addr.id} is in the schema, but is also in the block list"
+            err_msg = f"{dev_addr.id} is in the {SCHEMA}, but also in the {BLOCK_LIST}"
 
-    if not err_msg:
-        return gwy._get_device(dev_addr, ctl_addr=None, **kwargs)
+    if err_msg:
+        _LOGGER.error(f"%s: check the lists and the (cached) {SCHEMA}", err_msg)
 
-    _LOGGER.warning("%s: check the lists and the (cached) schema", err_msg)
+    return gwy._get_device(dev_addr, ctl_addr=ctl_addr, **kwargs)
 
 
 def load_system_schema(gwy, **kwargs) -> dict:
@@ -361,7 +381,8 @@ def load_system_schema(gwy, **kwargs) -> dict:
     [_get_device(gwy, addr(device_id)) for device_id in kwargs.pop(ATTR_ORPHANS, [])]
 
     if SCHEMA in kwargs:
-        _load_system_schema(gwy, kwargs[SCHEMA])
+        # ctl_id = kwargs.pop(ATTR_CONTROLLER)
+        _load_system_schema(gwy, schema=kwargs[SCHEMA])
         gwy.evo = gwy.system_by_id.get(kwargs[SCHEMA][ATTR_CONTROLLER])
 
     elif kwargs.get(MAIN_CONTROLLER):
@@ -378,15 +399,24 @@ def load_system_schema(gwy, **kwargs) -> dict:
     }
 
 
+def shrink_dict(_dict):
+    return {
+        k: shrink_dict(dict(v)) if isinstance(v, dict) else v
+        for k, v in _dict.items()
+        if bool(v)
+    }
+
+
 def _load_system_schema(gwy, schema) -> Tuple[dict, dict]:
+    # org_schema = schema
     schema = SYSTEM_SCHEMA(schema)
 
     ctl_id = schema[ATTR_CONTROLLER]
     profile = schema[ATTR_HTG_SYSTEM].get(ATTR_SYS_PROFILE)
     ctl = _get_device(gwy, addr(ctl_id), ctl_addr=addr(ctl_id), profile=profile)
 
-    if not ctl:
-        return
+    # if not ctl:
+    #     return
 
     htg_ctl_id = schema[ATTR_HTG_SYSTEM].get(ATTR_HTG_CONTROL)
     if htg_ctl_id:
@@ -394,7 +424,7 @@ def _load_system_schema(gwy, schema) -> Tuple[dict, dict]:
 
     dhw = schema.get(ATTR_DHW_SYSTEM, {})
     if dhw:
-        ctl._evo._set_dhw(ctl._evo._get_zone("HW"))
+        ctl._evo._set_dhw(ctl._evo._get_dhw())
 
         dhw_sensor_id = dhw.get(ATTR_DHW_SENSOR)
         if dhw_sensor_id:
@@ -427,7 +457,6 @@ def _load_system_schema(gwy, schema) -> Tuple[dict, dict]:
             zone._set_sensor(
                 _get_device(gwy, addr(sensor_id), ctl_addr=ctl, domain_id=zone_idx)
             )  # TODO: use domain_id=zone_idx or not
-
         if is_faked:
             zone.sensor._make_fake()
 
@@ -444,6 +473,13 @@ def _load_system_schema(gwy, schema) -> Tuple[dict, dict]:
         for ufc_id, _ in ufh_ctl_ids.items():
             _ = _get_device(gwy, addr(ufc_id), ctl_addr=ctl)
 
-    # assert schema == gwy.system_by_id[ctl_id].schema
+    if False and DEV_MODE:
+        import json
 
-    # return ctl._evo
+        src = json.dumps(shrink_dict(schema), sort_keys=True)
+        dst = json.dumps(shrink_dict(gwy.system_by_id[ctl_id].schema), sort_keys=True)
+        # assert dst == src, "They don't match!"
+        print(src)
+        print(dst)
+
+    return ctl
