@@ -4,14 +4,17 @@
 """RAMSES RF - The evohome-compatible zones."""
 
 import logging
-from abc import ABCMeta, abstractmethod
 from asyncio import Task
 from datetime import datetime as dt
 from datetime import timedelta as td
+from inspect import getmembers, isclass
+from sys import modules
+from types import SimpleNamespace
 from typing import Optional
 
 from .command import Command, Schedule
 from .const import (
+    _000C_DEVICE,
     ATTR_DEVICES,
     ATTR_DHW_SENSOR,
     ATTR_DHW_VALVE,
@@ -34,8 +37,9 @@ from .const import (
     ZoneMode,
     __dev_mode__,
 )
-from .devices import Device, Entity
+from .devices import BdrSwitch, Device, DhwSensor, Entity
 from .exceptions import CorruptStateError
+from .helpers import schedule_task
 
 # from .ramses import RAMSES_ZONES, RAMSES_ZONES_ALL
 
@@ -48,29 +52,31 @@ if DEV_MODE:
     _LOGGER.setLevel(logging.DEBUG)
 
 
-class ZoneBase(Entity, metaclass=ABCMeta):
+ZONE_CLASS = SimpleNamespace(
+    DHW="DHW",  # Stored HW (not a zone)
+    ELE="ELE",  # Electric
+    MIX="MIX",  # Mix valve
+    RAD="RAD",  # Radiator
+    UFH="UFH",  # Underfloor heating
+    VAL="VAL",  # Zone valve
+)
+
+
+class ZoneBase(Entity):
     """The Zone/DHW base class."""
 
-    DHW = "DHW"
-    ELE = "ELE"
-    MIX = "MIX"
-    RAD = "RAD"
-    UFH = "UFH"
-    VAL = "VAL"
+    # __zon_class__ = None  # NOTE: this would cause problems
 
     def __init__(self, evo, zone_idx) -> None:
-        _LOGGER.debug("Creating a Zone: %s_%s", evo.id, zone_idx)
+        _LOGGER.debug("Creating a Zone: %s_%s (%s)", evo, zone_idx, self.__class__)
         super().__init__(evo._gwy)
 
-        self.id = f"{evo.id}_{zone_idx}"
-        self.idx = zone_idx
+        self.id, self.idx = f"{evo.id}_{zone_idx}", zone_idx
+        self._evo, self._ctl = self._set_system(evo, zone_idx)
 
         self._name = None
         self._zone_type = None
 
-        self._evo = evo
-        self._ctl = evo._ctl
-
     def __repr__(self) -> str:
         return f"{self.id} ({self.heating_type})"
 
@@ -79,9 +85,30 @@ class ZoneBase(Entity, metaclass=ABCMeta):
             return NotImplemented
         return self.idx < other.idx
 
-    @abstractmethod
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
-        raise NotImplementedError
+    def _set_system(self, parent, zone_idx):
+        """Set the zone's parent system, after validating it."""
+
+        # these imports are here to prevent circular references
+        from .systems import System
+
+        try:
+            if zone_idx != "HW" and int(zone_idx, 16) >= parent.max_zones:
+                raise ValueError(f"{self}: invalid zone_idx {zone_idx} (> max_zones")
+        except (TypeError, ValueError):
+            raise TypeError(f"{self}: invalid zone_idx {zone_idx}")
+
+        if not isinstance(parent, System):
+            raise TypeError(f"{self}: parent must be a System, not {parent}")
+
+        if zone_idx != "HW":
+            if self.idx in parent.zone_by_idx:
+                raise LookupError(f"{self}: duplicate zone_idx: {zone_idx}")
+            parent.zone_by_idx[zone_idx] = self
+            parent.zones.append(self)
+
+        self._ctl = parent._ctl
+
+        return parent, parent._ctl
 
     def _handle_msg(self, msg) -> None:
         """Validate packets by verb/code."""
@@ -104,85 +131,9 @@ class ZoneBase(Entity, metaclass=ABCMeta):
         super()._send_cmd(code, dest, payload, **kwargs)
 
     @property
-    @abstractmethod
-    def schema(self) -> dict:
-        """Return the schema of the zone/DHW (e.g. sensor_id, zone_type)."""
-        raise NotImplementedError
-
-    @property
-    @abstractmethod
-    def params(self) -> dict:
-        """Return the configuration of the zone/DHW (e.g. min_temp, overrun)."""
-        raise NotImplementedError
-
-    @property
-    @abstractmethod
-    def status(self) -> dict:
-        """Return the current state of the zone/DHW (e.g. setpoint, temperature)."""
-        raise NotImplementedError
-
-    @property
-    @abstractmethod
-    def sensor(self) -> Device:
-        """Return the temperature sensor of the zone/DHW."""
-        raise NotImplementedError
-
-    @abstractmethod
-    def _set_sensor(self, value: Device) -> None:
-        """Set the temperature sensor for the zone (or for the DHW)."""
-        raise NotImplementedError
-
-    @property
-    @abstractmethod
-    def name(self) -> Optional[str]:
-        """Return the name of the zone/DHW."""
-        raise NotImplementedError
-
-    @property
-    @abstractmethod
-    def config(self) -> Optional[dict]:
-        """Return the configuration (parameters) of the zone/DHW."""
-        raise NotImplementedError
-
-    @property
-    @abstractmethod
-    def mode(self) -> Optional[dict]:
-        """Return the operating mode of the zone/DHW (mode, setpoint/active, until)."""
-        raise NotImplementedError
-
-    @property
-    @abstractmethod
-    def setpoint(self) -> Optional[float]:
-        """Return the target temperature of the zone/DHW."""
-        raise NotImplementedError
-
-    # @setpoint.setter
-    # @abstractmethod
-    # def setpoint(self, value: float) -> None:  # TODO: also mode, name attrs
-    #     """Set the target temperature of the zone/DHW."""
-    #     raise NotImplementedError
-
-    @property
-    @abstractmethod
-    def temperature(self) -> Optional[float]:
-        """Return the measured temperature of the zone/DHW."""
-        raise NotImplementedError
-
-    @property
-    def heating_type(self) -> str:
+    def heating_type(self) -> Optional[str]:
         """Return the type of the zone/DHW (e.g. electric_zone, stored_dhw)."""
-        return self._zone_type
-
-    # @abstractmethod
-    # def ._set_zone_type(self, value: str) -> None:
-    #     """Set the type of the zone/DHW (e.g. electric_zone, stored_dhw)."""
-    #     raise NotImplementedError
-
-    @property
-    @abstractmethod
-    def heat_demand(self) -> Optional[float]:
-        """Return the heat_demand of the zone/DHW."""
-        raise NotImplementedError
+        return self.__zon_class__
 
 
 class ZoneSchedule:  # 0404  # TODO: add for DHW
@@ -193,10 +144,10 @@ class ZoneSchedule:  # 0404  # TODO: add for DHW
 
         self._schedule = Schedule(self)
 
-    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+    # def _discover(self, discover_flag=DISCOVER_ALL) -> None:
 
-        if False and discover_flag & DISCOVER_STATUS:  # TODO: add back in
-            self._loop.create_task(self.get_schedule())  # 0404
+    #     if discover_flag & DISCOVER_STATUS:  # TODO: add back in
+    #         self._loop.create_task(self.get_schedule())  # 0404
 
     def _handle_msg(self, msg) -> bool:
         super()._handle_msg(msg)
@@ -240,16 +191,22 @@ class RelayDemand:  # 0008
 class DhwZone(ZoneBase):  # CS92A  # TODO: add Schedule
     """The DHW class."""
 
-    def __init__(self, ctl, sensor=None, dhw_valve=None, htg_valve=None) -> None:
-        super().__init__(ctl, "HW")
+    __zon_class__ = ZONE_CLASS.DHW
+
+    def __init__(
+        self, ctl, zone_idx="HW", sensor=None, dhw_valve=None, htg_valve=None
+    ) -> None:
+        super().__init__(ctl, zone_idx)
 
         ctl._set_dhw(self)
+        # if profile == ZONE_CLASS.DHW and evo.dhw is None:
+        #     evo.dhw = zone
 
         self._dhw_sensor = None
         self._dhw_valve = None
         self._htg_valve = None
 
-        self._zone_type = Zone.DHW
+        self._zone_type = ZONE_CLASS.DHW
 
         self._dhw_mode = None
         self._dhw_params = None
@@ -266,25 +223,32 @@ class DhwZone(ZoneBase):  # CS92A  # TODO: add Schedule
         if htg_valve:
             self._set_htg_valve(htg_valve)
 
-    def _discover(self, discover_flags=DISCOVER_ALL) -> None:
+    def _discover(self, discover_flag=DISCOVER_ALL) -> None:
         # super()._discover(discover_flag=discover_flag)
 
         # if False and __dev_mode__ and self.idx == "FA":  # dev/test code
         #     self.async_set_override(state="On")
 
-        if discover_flags & DISCOVER_SCHEMA:
-            [  # 000C: find the DHW relay(s), if any, see: CODE_000C_DEVICE_TYPE
+        if discover_flag & DISCOVER_SCHEMA:
+            [  # 000C: find the DHW relay(s), if any, see: _000C_DEVICE_TYPE
                 self._send_cmd("000C", payload=dev_type)
-                for dev_type in ("000D", "000E", "010E")  # for DHW sensor, relay(s)
+                for dev_type in (
+                    f"00{_000C_DEVICE.DHW_SENSOR}",
+                    f"00{_000C_DEVICE.DHW}",
+                    f"01{_000C_DEVICE.DHW}",
+                )
             ]
 
-        if discover_flags & DISCOVER_PARAMS:
-            for code in ("10A0",):
-                self._send_cmd(code, payload="00")  # payload="00" (or "0000"), not "FA"
+        if discover_flag & DISCOVER_PARAMS:
+            self._gwy.send_cmd(
+                Command.get_dhw_params(self._ctl.id), period=td(hours=12)
+            )
 
-        if discover_flags & DISCOVER_STATUS:
-            for code in ("1260", "1F41"):
-                self._send_cmd(code, payload="00")  # payload="00" or "0000", not "FA"
+        if discover_flag & DISCOVER_STATUS:
+            self._gwy.send_cmd(Command.get_dhw_mode(self._ctl.id), period=td(hours=12))
+            self._gwy.send_cmd(
+                Command.get_dhw_temp(self._ctl.id), period=td(minutes=30)
+            )
 
     def _handle_msg(self, msg) -> bool:
         super()._handle_msg(msg)
@@ -304,31 +268,36 @@ class DhwZone(ZoneBase):  # CS92A  # TODO: add Schedule
         elif msg.code == "1F41":
             self._dhw_mode = msg
 
-    def _set_sensor(self, device: Device) -> None:
-        """Set the temp sensor for this DHW system (07: only)."""
-
-        if self._dhw_sensor is device:
-            return
-        elif self._dhw_sensor is not None:
+    def _set_dhw_device(self, new_dev, old_dev, attr_name, dev_class, domain_id):
+        if old_dev is new_dev:
+            return old_dev
+        if old_dev is not None:
             raise CorruptStateError(
-                f"{ATTR_ZONE_SENSOR} shouldn't change: {self._dhw_sensor} to {device}"
+                f"{self} changed {attr_name}: {old_dev} to {new_dev}"
             )
 
-        if not isinstance(device, Device) or device.type != "07":
-            raise TypeError(f"{ATTR_ZONE_SENSOR} can't be: {device}")
+        if not isinstance(new_dev, dev_class):
+            raise TypeError(f"{self}: {attr_name} can't be {dev_class}")
 
-        if self._dhw_sensor is None:
-            self._dhw_sensor = device
-            device._set_parent(self, domain="FA")
+        new_dev._set_parent(self, domain=domain_id)
+        return new_dev
+
+    def _set_sensor(self, device: DhwSensor) -> None:
+        """Set the temp sensor for this DHW system (07: only)."""
+
+        # 07:38:39.124 047 RQ --- 07:030741 01:102458 --:------ 10A0 006 00181F0003E4
+        # 07:38:39.140 062 RP --- 01:102458 07:030741 --:------ 10A0 006 0018380003E8
+
+        self._dhw_sensor = self._set_dhw_device(
+            device, self._dhw_sensor, ATTR_ZONE_SENSOR, DhwSensor, "FA"
+        )
 
     @property
-    def sensor(self) -> Device:  # self._dhw_sensor
+    def sensor(self) -> DhwSensor:  # self._dhw_sensor
         return self._dhw_sensor
 
-    def _set_dhw_valve(self, device: Device) -> None:
-        """Set the hotwater valve relay for this DHW system (13: only)."""
-
-        """Blah it now.
+    def _set_dhw_valve(self, device: BdrSwitch) -> None:  # self._dhw_valve
+        """Set the hotwater valve relay for this DHW system (13: only).
 
         Check and Verb the DHW sensor (07:) of this system/CTL (if there is one).
 
@@ -339,48 +308,23 @@ class DhwZone(ZoneBase):  # CS92A  # TODO: add Schedule
         The I/1260 is not to/from a controller, so is not useful.
         """  # noqa: D402
 
-        # 07:38:39.124 047 RQ --- 07:030741 01:102458 --:------ 10A0 006 00181F0003E4
-        # 07:38:39.140 062 RP --- 01:102458 07:030741 --:------ 10A0 006 0018380003E8
-
-        # if "10A0" in self._msgs:
-        #     return self._msgs["10A0"].dst.addr
-        if not isinstance(device, Device) or device.type != "13":
-            raise TypeError(f"{ATTR_DHW_VALVE} can't be: {device}")
-
-        if self._dhw_valve is not None:
-            if self._dhw_valve is device:
-                return
-            raise CorruptStateError(
-                f"{ATTR_DHW_VALVE} shouldn't change: {self._dhw_valve} to {device}"
-            )
-
-        if self._dhw_valve is None:
-            self._dhw_valve = device
-            device._set_parent(self, domain="FA")
+        self._dhw_valve = self._set_dhw_device(
+            device, self._dhw_valve, ATTR_DHW_VALVE, BdrSwitch, "FA"
+        )
 
     @property
-    def hotwater_valve(self) -> Device:  # self._dhw_valve
+    def hotwater_valve(self) -> BdrSwitch:  # self._dhw_valve
         return self._dhw_valve
 
-    def _set_htg_valve(self, device: Device) -> None:  # self._htg_valve
+    def _set_htg_valve(self, device: BdrSwitch) -> None:  # self._htg_valve
         """Set the heating valve relay for this DHW system (13: only)."""
 
-        if not isinstance(device, Device) or device.type != "13":
-            raise TypeError(f"{ATTR_DHW_VALVE_HTG} can't be: {device}")
-
-        if self._htg_valve is not None:
-            if self._htg_valve is device:
-                return
-            raise CorruptStateError(
-                f"{ATTR_DHW_VALVE_HTG} shouldn't change: {self._htg_valve} to {device}"
-            )
-
-        if self._htg_valve is None:
-            self._htg_valve = device
-            device._set_parent(self, domain="F9")
+        self._htg_valve = self._set_dhw_device(
+            device, self._htg_valve, ATTR_DHW_VALVE_HTG, BdrSwitch, "F9"
+        )
 
     @property
-    def heating_valve(self) -> Device:  # self._htg_valve
+    def heating_valve(self) -> BdrSwitch:  # self._htg_valve
         return self._htg_valve
 
     @property
@@ -477,7 +421,9 @@ class DhwZone(ZoneBase):  # CS92A  # TODO: add Schedule
 class Zone(ZoneSchedule, ZoneBase):
     """The Zone base class."""
 
-    def __init__(self, ctl, zone_idx, sensor=None, actuators=None) -> None:
+    __zon_class__ = None  # Unknown
+
+    def __init__(self, evo, zone_idx, sensor=None, actuators=None) -> None:
         """Create a zone.
 
         The type of zone may not be known at instantiation. Even when it is known, zones
@@ -486,13 +432,10 @@ class Zone(ZoneSchedule, ZoneBase):
 
         In addition, an electric zone may subsequently turn out to be a zone valve zone.
         """
-        # _LOGGER.debug("Creating a Zone: %s (%s)", zone_idx, self.__class__)
-        super().__init__(ctl, zone_idx)
+        if int(zone_idx, 16) >= evo.max_zones:
+            raise ValueError(f"Invalid zone idx: {zone_idx} (exceeds max_zones)")
 
-        # self.id = f"{ctl.id}_{zone_idx}"
-        ctl.zones.append(self)
-        ctl.zone_by_idx[zone_idx] = self
-        # ctl.zone_by_name[self.name] = self
+        super().__init__(evo, zone_idx)
 
         self.devices = []
         self.device_by_id = {}
@@ -505,13 +448,13 @@ class Zone(ZoneSchedule, ZoneBase):
         self._temperature = None
         self._zone_config = None
 
-        # these needed here, as we can't use __init__
+        # these needed here, as we can't use those __init__()s
         self._mix_config = None  # MIX
         self._ufh_setpoint = None  # UFH
         self._window_open = None  # RAD (or: ALL)
         self._actuator_state = None  # ELE, ZON
 
-        self._schedule = Schedule(self)
+        # self._schedule = Schedule(self)  # TODO:
 
         if sensor:
             self._set_sensor(sensor)
@@ -521,17 +464,19 @@ class Zone(ZoneSchedule, ZoneBase):
 
         # TODO: add code to determine zone type if it doesn't have one, using 0005s
         if discover_flag & DISCOVER_SCHEMA:
-            [  # 000C: find the sensor and the actuators, if any
-                self._send_cmd("000C", payload=f"{self.idx}{dev_type}")
-                for dev_type in ("00", "04")  # CODE_000C_ZONE_TYPE
-            ]
+            self._send_cmd("000C", payload=f"{self.idx}{_000C_DEVICE.ALL}")
+            self._send_cmd("000C", payload=f"{self.idx}{_000C_DEVICE.ALL_SENSOR}")
 
-        if discover_flag & DISCOVER_PARAMS:  # every 4h
-            self._gwy.send_cmd(Command.get_zone_config(self._ctl.id, self.idx))  # 000A
-            self._gwy.send_cmd(Command.get_zone_name(self._ctl.id, self.idx))  # 0004
+        if discover_flag & DISCOVER_PARAMS:
+            self._gwy.send_cmd(Command.get_zone_config(self._ctl.id, self.idx))
+            self._gwy.send_cmd(
+                Command.get_zone_name(self._ctl.id, self.idx), period=td(hours=4)
+            )
 
         if discover_flag & DISCOVER_STATUS:  # every 1h, CTL will not respond to a 3150
-            self._gwy.send_cmd(Command.get_zone_mode(self._ctl.id, self.idx))
+            self._gwy.send_cmd(
+                Command.get_zone_mode(self._ctl.id, self.idx), period=td(minutes=30)
+            )
             self._gwy.send_cmd(Command.get_zone_temperature(self._ctl.id, self.idx))
             self._gwy.send_cmd(Command.get_zone_window_state(self._ctl.id, self.idx))
 
@@ -589,20 +534,20 @@ class Zone(ZoneSchedule, ZoneBase):
     def sensor(self) -> Device:
         return self._sensor
 
-    def _set_sensor(self, device: Device):  # self._sensor
+    def _set_sensor(self, device: Device) -> None:  # self._sensor
         """Set the temp sensor for this zone (one of: 01:, 03:, 04:, 12:, 22:, 34:)."""
 
         if self._sensor is device:
             return
-        elif self._sensor is not None:
-            _LOGGER.error("Changing the Zone sensor for %s to %s", self, device)
-            # raise CorruptStateError(
-            #     f"{ATTR_ZONE_SENSOR} shouldn't change: {self._sensor} to {device}"
-            # )
+        if self._sensor is not None:
+            raise CorruptStateError(
+                f"{self} changed {ATTR_ZONE_SENSOR}: {self._sensor} to {device}"
+            )
 
         sensor_types = ("00", "01", "03", "04", "12", "22", "34")
         if not isinstance(device, Device) or device.type not in sensor_types:
-            raise TypeError(f"{ATTR_ZONE_SENSOR} can't be: {device}")
+            # TODO: or not hasattr(device, "temperature")
+            raise TypeError(f"{self}: {ATTR_ZONE_SENSOR} can't be: {device}")
 
         self._sensor = device
         device._set_parent(self)  # , domain=self.idx)
@@ -651,23 +596,20 @@ class Zone(ZoneSchedule, ZoneBase):
         """
 
         _type = ZONE_TYPE_SLUGS.get(zone_type, zone_type)
-        if _type not in ZONE_CLASSES:
-            raise ValueError(f"Not a known zone type: {zone_type}")
+        if _type not in ZONE_BY_CLASS_ID:
+            raise ValueError(f"Not a known zone_type: {zone_type}")
 
-        if (
-            self._zone_type is not None
-            and self._zone_type != _type
-            and (self._zone_type != "ELE" and _type != "VAL")
+        if self._zone_type == _type:
+            return
+        if self._zone_type is not None and (
+            self._zone_type != "ELE" and _type != "VAL"
         ):
             raise CorruptStateError(
-                f"Zone {self} has a mismatched type: "
-                f"old={self._zone_type}, new={_type}"
+                f"{self} changed zone_type: {self._zone_type} to {_type}"
             )
 
         self._zone_type = _type
-        self.__class__ = ZONE_CLASSES[_type]
-        _LOGGER.debug("Zone %s: type now set to %s", self.id, self._zone_type)
-
+        self.__class__ = ZONE_BY_CLASS_ID[_type]
         self._discover()  # TODO: needs tidyup (ref #67)
 
     @property
@@ -858,12 +800,15 @@ class Zone(ZoneSchedule, ZoneBase):
 class EleZone(RelayDemand, Zone):  # BDR91A/T  # TODO: 0008/0009/3150
     """For a small electric load controlled by a relay (never calls for heat)."""
 
+    __zon_class__ = ZONE_CLASS.ELE
+
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+        # NOTE: we create, then promote, so shouldn't super()
         # super()._discover(discover_flag=discover_flag)
-        if discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd("000C", payload=f"{self.idx}11")
+        if False and discover_flag & DISCOVER_SCHEMA:
+            self._send_cmd("000C", payload=f"{self.idx}{_000C_DEVICE.ELE}")
 
     def _handle_msg(self, msg) -> bool:
         super()._handle_msg(msg)
@@ -887,12 +832,15 @@ class MixZone(Zone):  # HM80  # TODO: 0008/0009/3150
     Note that HM80s are listen-only devices.
     """
 
+    __zon_class__ = ZONE_CLASS.MIX
+
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+        # NOTE: we create, then promote, so shouldn't super()
         # super()._discover(discover_flag=discover_flag)
-        if discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd("000C", payload=f"{self.idx}0B")
+        if False and discover_flag & DISCOVER_SCHEMA:
+            self._send_cmd("000C", payload=f"{self.idx}{_000C_DEVICE.MIX}")
 
         if discover_flag & DISCOVER_PARAMS:
             self._gwy.send_cmd(Command.get_mix_valve_params(self._ctl.id, self.idx))
@@ -915,23 +863,29 @@ class MixZone(Zone):  # HM80  # TODO: 0008/0009/3150
 class RadZone(Zone):  # HR92/HR80
     """For radiators controlled by HR92s or HR80s (will also call for heat)."""
 
+    __zon_class__ = ZONE_CLASS.RAD
+
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+        # NOTE: we create, then promote, so shouldn't super()
         # super()._discover(discover_flag=discover_flag)
-        if discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd("000C", payload=f"{self.idx}08")
+        if False and discover_flag & DISCOVER_SCHEMA:
+            self._send_cmd("000C", payload=f"{self.idx}{_000C_DEVICE.RAD}")
 
 
 class UfhZone(Zone):  # HCC80/HCE80  # TODO: needs checking
     """For underfloor heating controlled by an HCE80/HCC80 (will also call for heat)."""
 
+    __zon_class__ = ZONE_CLASS.UFH
+
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+        # NOTE: we create, then promote, so shouldn't super()
         # super()._discover(discover_flag=discover_flag)
-        if discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd("000C", payload=f"{self.idx}09")
+        if False and discover_flag & DISCOVER_SCHEMA:
+            self._send_cmd("000C", payload=f"{self.idx}{_000C_DEVICE.UFH}")
 
     def _handle_msg(self, msg) -> bool:
         super()._handle_msg(msg)
@@ -951,12 +905,15 @@ class UfhZone(Zone):  # HCC80/HCE80  # TODO: needs checking
 class ValZone(EleZone):  # BDR91A/T
     """For a motorised valve controlled by a BDR91 (will also call for heat)."""
 
+    __zon_class__ = ZONE_CLASS.VAL
+
     # def __init__(self, *args, **kwargs) -> None:  # can't use this here
 
     def _discover(self, discover_flag=DISCOVER_ALL) -> None:
+        # NOTE: we create, then promote, so shouldn't super()
         # super()._discover(discover_flag=discover_flag)
-        if discover_flag & DISCOVER_SCHEMA:
-            self._send_cmd("000C", payload=f"{self.idx}0A")
+        if False and discover_flag & DISCOVER_SCHEMA:
+            self._send_cmd("000C", payload=f"{self.idx}{_000C_DEVICE.VAL}")
 
     @property
     def heat_demand(self) -> Optional[float]:  # 0008 (NOTE: not 3150)
@@ -964,11 +921,26 @@ class ValZone(EleZone):  # BDR91A/T
         return self.relay_demand
 
 
-ZONE_CLASSES = {
-    Zone.DHW: DhwZone,
-    Zone.ELE: EleZone,
-    Zone.MIX: MixZone,
-    Zone.RAD: RadZone,
-    Zone.VAL: ValZone,
-    Zone.UFH: UfhZone,
-}
+CLASS_ATTR = "__zon_class__"
+ZONE_BY_CLASS_ID = {
+    getattr(c[1], CLASS_ATTR): c[1]
+    for c in getmembers(
+        modules[__name__],
+        lambda m: isclass(m) and m.__module__ == __name__ and hasattr(m, CLASS_ATTR),
+    )
+}  # e.g. "RAD": RadZone
+
+
+def create_zone(evo, zone_idx, profile=None, **kwargs) -> Zone:
+    """Create a zone, and optionally perform discovery & start polling."""
+
+    if profile is None:
+        profile = ZONE_CLASS.DHW if zone_idx == "HW" else None
+
+    zone = ZONE_BY_CLASS_ID.get(profile, Zone)(evo, zone_idx, **kwargs)
+
+    if not evo._gwy.config.disable_discovery:
+        schedule_task(1, zone._discover, discover_flag=DISCOVER_SCHEMA)
+        schedule_task(4, zone._discover, discover_flag=DISCOVER_PARAMS)
+        schedule_task(7, zone._discover, discover_flag=DISCOVER_STATUS)
+    return zone
